
#include <stdbool.h>

/** An interval is made of two integer bounds.

    Member lo is the lower bound, member up is the upper bound.

    An integer x belongs to the interval i if i.lo <= x < i.up, i.e.,
    all intervals are open on the right.
*/
typedef struct {
    int lo; /**< the lower bound (included) */
    int up; /**< the upper bound (excluded) */
} interval;

/** \fn bool interval_big (interval inter)
    \brief Indicates whether an interval is big.

    An interval is big if it contains more than one integer.

    \param[in] inter The interval to test.
    \returns true iff the interval is big.
*/
bool interval_big (interval inter);

/** \fn bool interval_in (interval inter, int value)
    \brief Indicates whether the value belongs to the interval.

    Integer value belongs to the interval if it is greater than or
    equal to the lower bound, and less than the upper bound.

    \param inter The interval.
    \param value The integer to test.
    \returns true iff the value belongs to the interval.
*/
bool interval_in (interval inter, int value);

/** \fn bool interval_intersect (interval inter1, interval inter2)
    \brief Indicates whether both intervals intersect.

    Two intervals intersect if there is at least one integer that
    belongs to both of them.

    \param inter1 The first interval.
    \param inter2 The second interval.
    \returns true iff both intervals intersect.
*/
bool interval_intersect (interval inter1, interval inter2);

/** \fn bool interval_parse (const char * s, interval * intv, char * err, size_t errsize)
    \brief Reads an interval from a string, or indicates error.

    The string must have one of two formats:

    - either "(<snum>,<snum>)" (e.g., "(-123,456)"): this form gives
      the two bounds, in order;

    - or "(<snum>:<unum>)" (e.g., "(-123:246)"): this form gives the
      lower bound and the width of the interval (the latter cannot
      include a sign).

    In these descriptions, "<snum>" represents an optionnaly signed
    integer (e.g., "123", "-123", or "+123"), and "<unum>" an unsigned
    integer (e.g., "123").

    If the string is correct, the interval pointed to by intv is
    filled with the extracted integers, and err and errsize are left
    untouched.

    If the string is incorrect, a suitable error message is written
    into the array pointed to by err. No more than errsize characters
    (including the terminating nul character) will be written there.
    Actually, both err and errsize are passed directly to snprintf.
    The interval pointed to by intv is left untouched.

    \param s The string to parse.
    \param intv A pointer to the interval to initialize.
    \param err A string to hold the error message.
    \param errsize The number of characters available in err.
    \returns true iff the interval has been successfully initialized.
*/
bool interval_parse (const char * s, interval * intv,
                     char * err, size_t errsize);
